from datetime import datetime, timedelta

from common.utils import logging
from common.utils.baseline.core import BaselineService
from common.utils.checkpoint.core import CheckpointService
from common.utils.event.core import EventService
from common.utils.flow.core import FlowService
from common.utils.lookup.core import LookupService
from common.utils.userconfig.core import UserConfigService
from verticalportscan.detector.core import VerticalPortScan


def main():
    """ Main function for detector

    Runs the detector for the current day, if it's 2am, run a second time
    for the previous day
    """

    now = datetime.now()
    vscan = init_vscan_with_time(now)

    logger.info("starting vertical port scan for today")
    vscan.run()
    logger.info("ending vertical port scan for today")

    if now.hour == 2:
        logger.info("running during hour 2, triggering run for yesterday")
        end_of_yesterday = datetime.combine(
            now.date() - timedelta(days=1), datetime.max.time())
        vscan = init_vscan_with_time(end_of_yesterday)

        logger.info("starting vertical port scan for yesterday")
        vscan.run()
        logger.info("ending vertical port scan for yesterday")


def init_vscan_with_time(time: datetime) -> VerticalPortScan:
    flow_service = FlowService()
    return VerticalPortScan(
        BaselineService(),
        CheckpointService(),
        UserConfigService(),
        EventService(),
        flow_service,
        LookupService(flow_service),
        time
    )


if __name__ == "__main__":
    logger = logging.get_common_logger(__file__)
    logger.info("initializing vertical port scan detector")
    main()
